@@ -7,6 +7,10 @@
 |
 | The full license is in the file LICENSE, distributed with this software.
 |----------------------------------------------------------------------------*/
+/**
+ * @packageDocumentation
+ * @module commands
+ */
 import { ArrayExt } from '@lumino/algorithm';
 
 import {
@@ -1461,8 +1465,7 @@ namespace Private {
   const MAC_DISPLAY: { [key: string]: string } = {
     Backspace: '⌫',
     Tab: '⇥',
-    // U+FE0E prevents the "enter" character from rendering in color as an emoji
-    Enter: '↩\uFE0E',
+    Enter: '⏎',
     Shift: '⇧',
     Ctrl: '⌃',
     Alt: '⌥',
